@@ -57,7 +57,8 @@ class Model(object):
 
     def __init__(self, dbe, comps, phase_name, parameters=None):
         # Constrain possible components to those within phase's d.o.f
-        possible_comps = set([x.upper() for x in comps])
+        possible_comps = {x.upper() for x in comps}
+        comps = sorted(comps, key=str)
         self.components = set()
         self.constituents = []
         self.phase_name = phase_name.upper()
@@ -80,10 +81,16 @@ class Model(object):
 
         # Convert string symbol names to sympy Symbol objects
         # This makes xreplace work with the symbols dict
-        symbols = dict([(Symbol(s), val) for s, val in dbe.symbols.items()])
+        symbols = {Symbol(s): val for s, val in dbe.symbols.items()}
 
+        def wrap_symbol(obj):
+            if isinstance(obj, Symbol):
+                return obj
+            else:
+                return Symbol(obj)
         if parameters is not None:
-            symbols.update([(Symbol(s), val) for s, val in parameters.items()])
+            symbols.update([(wrap_symbol(s), val) for s, val in parameters.items()])
+        self._symbols = {wrap_symbol(key): value for key, value in symbols.items()}
 
         self.models = OrderedDict()
         self.build_phase(dbe)
@@ -331,8 +338,8 @@ class Model(object):
                     comp_symbols = \
                         [
                             v.SiteFraction(phase.name, subl_index, comp)
-                            for comp in set(phase.constituents[subl_index])\
-                                .intersection(self.components)
+                            for comp in sorted(set(phase.constituents[subl_index])\
+                                .intersection(self.components))
                         ]
                     mixing_term *= Add(*comp_symbols)
                 else:
@@ -424,8 +431,6 @@ class Model(object):
         ideal_mixing_term = S.Zero
         for subl_index, sublattice in enumerate(phase.constituents):
             active_comps = set(sublattice).intersection(self.components)
-            if len(active_comps) == 1:
-                continue # no mixing if only one species in sublattice
             ratio = site_ratios[subl_index]
             for comp in active_comps:
                 sitefrac = \
@@ -506,7 +511,7 @@ class Model(object):
         self.TC = self.curie_temperature = tc
         #print(tc)
         # 0.01 used to prevent singularity
-        tau = v.T / (tc + 0.01)
+        tau = v.T / (tc + 1e-9)
 
         # define model parameters
         p = phase.model_hints['ihj_magnetic_structure_factor']
@@ -572,8 +577,8 @@ class Model(object):
         neel_temp = \
             self.redlich_kister_sum(phase, param_search, nt_param_query)
 
-        self.TC = self.curie_temperature = curie_temp.xreplace(dbe.symbols)
-        self.NT = self.neel_temperature = neel_temp.xreplace(dbe.symbols)
+        self.TC = self.curie_temperature = curie_temp.subs(self._symbols)
+        self.NT = self.neel_temperature = neel_temp.subs(self._symbols)
 
         tau_curie = v.T / curie_temp
         tau_curie = tau_curie.xreplace({zoo: 1.0e10})
@@ -691,7 +696,7 @@ class Model(object):
         disordered_phase_name = phase.model_hints.get('disordered_phase', None)
         if phase.name != ordered_phase_name:
             return S.Zero
-        disordered_model = self.__class__(dbe, self.components,
+        disordered_model = self.__class__(dbe, sorted(self.components),
                                           disordered_phase_name)
         constituents = [sorted(set(c).intersection(self.components)) \
                 for c in dbe.phases[ordered_phase_name].constituents]
@@ -782,7 +787,7 @@ class TestModel(Model):
     None yet.
     """
     def __init__(self, dbf, comps, phase, solution=None, kmax=None):
-        self.components = set([x.upper() for x in comps])
+        self.components = {x.upper() for x in comps}
         if 'VA' in self.components:
             raise ValueError('Vacancies are unsupported in TestModel')
         self.models = dict()
